# Operator - Sign PYTH-SPL Transaction

## Requirements

* Online machine

* [High Visibility Storage](TODO): plastic container or bag that's used to keep items while not in use in a visible location like the middle of a desk.

* [Operator PGP key pairs](../../key-types.md#operator-pgp-keypair) 

{{ #include ../../../../operator-requirements.md:requirements }}

## Procedure

1. Verify all transactions for the ceremony in the `ceremonies` repository, ensuring that all the transactions are properly signed by the proposer and the approver using PGP keys which have been checked into ceremonies repository.

1. Copy the transactions and signatures to an SD card

1. Enter the designated location with the quorum of operators and all required equipment

1. Lock access to the location - there should be no inflow or outflow of people during the ceremony

1. Retrieve sealed Air-Gapped bundle and polaroid from locked storage

### Unsealing Tamper Proofing

{{ #include ../../../../../../component-documents/tamper-evidence-methods.md:vsbwf-procedure-unsealing}}

1. Place all contents except for the laptop into High Visibility Storage

### Ceremony

1. Turn on online machine

1. Once booted, run `icepick workflow sol broadcast --nonce-address=<nonce_address>` command

    - [ ] TODO find a place for the nonce_address in the ceremony repo or airgapOS

    * Plug in "Transaction" SD card

    * Await completion message before removing "Transaction" SD card

    * This command will set the computer into "awaiting mode", which will broadcast the signed transaction from the SD card once it's plugged back in

1. Plug in SD card labelled "AirgapOS" into the air-gapped machine

1. Boot the computer  

1. Unplug the "AirgapOS" SD card and place it in High Visibility Storage

1. Plug in SD card labelled "Keychain"

1. Use detached signatures of the keys on the SD cards to verify the pub certs. Each operator should verify that the each key in the Keychain has been signed by that operator's key.

    1. Plug in a smart card with Operator PGP Key

    1. Use the following script to check that a signature by the plugged in key exists for all the public certificates found on the Keychain 
    - [ ] TODO move this to ceremony SD card or airgapOS
    ```
    #!/bin/bash

    set -eu -o pipefail

    DIRECTORY="$1"

    if ! compgen -G "$DIRECTORY/*.asc" > /dev/null; then
    echo "No .asc files found in the directory."
    exit 1
    fi

    if ! gpg --card-status > /dev/null 2>&1; then
    echo "No smart card detected. Please insert a smart card."
    exit 1
    fi

    smart_card_id=$(gpg --card-status | grep sec | cut -d'/' -f2 | cut -d' ' -f1)

    for asc_file in "$DIRECTORY"/*.asc; do
    gpg --import "$asc_file"
    done

    for asc_file in "$DIRECTORY"/*.asc; do
    sig_exists="false"

    for sig_file in "$DIRECTORY"/*.asc.sig; do
        sigfile_basename=$(basename "$sig_file" .asc.sig)
        ascfile_basename=$(basename "$asc_file" .asc)

        if [[ "$sigfile_basename" != "$ascfile_basename" ]]; then
            continue
        fi

        sig_key_id=$(gpg --verify "$sig_file" 2>&1 | grep 'Primary key fingerprint' | cut -d' ' -f4- | tr -d ' ')
        trimmed_key_id="${sig_key_id: -16}"

        if [[ "$trimmed_key_id" == "$smart_card_id" ]]; then
            sig_exists="true"
            break
        fi
    done

    if [[ "$sig_exists" == "false" ]]; then
        printf "\nWARNING: Signature for %s by operator key %s does not exist\n" "$asc_file" "$smart_card_id"
    fi
    done

    ```

1. Ensure that the script doesn't output any "WARNING" messages to the console. If it does, abort the ceremony and initiate incident response.
    
1. Unplug the "Keychain" SD card and place it in High Visibility Storage

1. Insert SD card labelled "Shardfile"

1. If no prompt appears, run the command:

    * `keyfork recover shard --daemon`

1. Follow on screen prompts

1. Unplug SD card labelled "Shardfile" and place it in High Visibility Storage

1. Retrieve the "Transaction" SD card from High Visibility Storage and plug in the "Transaction" SD card

1. For each transaction, verify that the signature is made by trusted keys that are loaded in the gpg keyring:
    - [ ] TODO: move this to ceremony SD card
    ```
    #!/bin/bash

    DIRECTORY=$1

    declare -a key_ids

    while IFS= read -r line; do
        key_id=$(echo "$line" | awk -F: '/^pub/{print $5}')
        if [[ -n "$key_id" ]]; then
            key_ids+=("$key_id")
        fi
    done < <(gpg --list-keys --with-colons)

    check_key_id() {
        local search_key_id="$1"
        for id in "${key_ids[@]}"; do
            if [[ "$id" == "$search_key_id" ]]; then
                return 0 
            fi
        done
        return 1 
    }

    for tx in "$DIRECTORY"/*.json; do
        basename=$(basename "$tx" .json)
        number_of_sigs=0
        tx_sig="$DIRECTORY/$basename.json.sig"

        if [[ ! -f "$tx_sig" ]]; then
            echo "WARNING: No signature file found for transaction $tx."
            continue
        fi

        sig_key_id=$(gpg --verify "$tx_sig" 2>&1 | grep 'Primary key fingerprint' | cut -d' ' -f4- | tr -d ' ')
        trimmed_key_id="${sig_key_id: -16}"

        if check_key_id "$trimmed_key_id"; then
            ((number_of_sigs++))
        else
            echo "Key ID $trimmed_key_id not found in key_ids array."
        fi    

        if (( number_of_sigs < 2 )); then
            echo "WARNING: Insufficient signatures ($number_of_sigs) for transaction $tx."
        fi
    done
    ```


1. Run the `icepick` command with the transaction payload

    * `icepick workflow sol transfer-token --input-file=<payload.json>` 

    * Follow on screen prompts

1. Unplug the "Transaction" SD card and place it in High Visibility Storage

### Broadcast Transaction

1. Plug the "Transaction" SD card into online machine

1. The still running broadcast command on the online machine will broadcast the transaction automatically

1. The url that's found in the response after a successful broadcast should be reviewed and committed to the ceremony repository

### Finalization

1. Shut down online machine

1. Shut down the air gapped machine

#### Sealing

1. Gather all the original items that were in the air-gapped bundle:

    * Air-gapped computer

    * AirgapOS SD card

    * Shardfile SD card

    * Keychain SD card

{{ #include ../../../../../../component-documents/tamper-evidence-methods.md:vsbwf-procedure-sealing}}

